package net.sergeych.bintools

import java.io.InputStream
import java.io.OutputStream
import java.nio.file.Path
import kotlin.io.path.*

class FileDataProvider(val folder: Path) : DataProvider {

    class Source(private val input: InputStream) : DataSource {
        override fun readByte(): Byte {
            val b = input.read()
            if (b < 0) throw DataSource.EndOfData()
            return b.toByte()
        }

        override fun readBytes(size: Int): ByteArray {
            return input.readNBytes2(size).also {
                if (it.size < size) throw DataSource.EndOfData()
            }
        }
    }

    class Sink(private val out: OutputStream) : DataSink {
        override fun writeByte(data: Byte) {
            out.write(data.toInt())
        }
    }

    override fun <T> read(name: String, f: (DataSource) -> T): T = folder.resolve(name).inputStream().buffered().use {
        f(Source(it))
    }

    override fun write(name: String, f: (DataSink) -> Unit) {
        folder.resolve(name).outputStream().use { f(Sink(it)) }
    }

    override fun delete(name: String) {
        println("file: $folder -- $name")
        folder.resolve(name).deleteExisting()
    }

    override fun list(): List<String> = folder
        .listDirectoryEntries()
        .filter { it.isRegularFile() && it.isReadable() }
        .map { it.name }
}

/**
 * Compatibility with Java8 and android. Read up to N bytes
 */
fun InputStream.readNBytes2(size: Int): ByteArray {
    val result = ByteArray(size)
    val len = read(result)
    return if (len < size)
        result.sliceArray(0..<len)
    else
        result
}